@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------------
    libconfig - A library for processing structured configuration files
-   Copyright (C) 2005-2014  Mark A Lindner
+   Copyright (C) 2005-2015  Mark A Lindner
 
    This file is part of libconfig.
 
@@ -26,6 +26,7 @@
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
+#include <dirent.h>
 
 #define STRING_BLOCK_SIZE 64
 #define CHUNK_SIZE 32
@@ -92,43 +93,26 @@ const char **scanctx_cleanup(struct scan_context *ctx,
 /* ------------------------------------------------------------------------- */
 
 FILE *scanctx_push_include(struct scan_context *ctx, void *buffer,
-                           const char **error)
+                            const char *file, const char **error)
 {
   FILE *fp = NULL;
-  const char *file;
-  char *full_file = NULL;
-
-  *error = NULL;
 
   if(ctx->depth == MAX_INCLUDE_DEPTH)
   {
     *error = err_include_too_deep;
     return(NULL);
   }
-
-  file = scanctx_take_string(ctx);
-  if(ctx->config->include_dir)
-  {
-    full_file = (char *)malloc(strlen(ctx->config->include_dir) + strlen(file)
-                               + 2);
-    strcpy(full_file, ctx->config->include_dir);
-    strcat(full_file, FILE_SEPARATOR);
-    strcat(full_file, file);
-  }
-
-  fp = fopen(full_file ? full_file : file, "rt");
-  free((void *)full_file);
-
+  fp = fopen(file, "rt");
   if(fp)
   {
     ctx->streams[ctx->depth] = fp;
     ctx->files[ctx->depth] = __scanctx_add_filename(ctx, file);
     ctx->buffers[ctx->depth] = buffer;
     ++(ctx->depth);
+      *error = NULL;
   }
   else
   {
-    free((void *)file);
     *error = err_bad_include;
   }
 
@@ -137,6 +121,40 @@ FILE *scanctx_push_include(struct scan_context *ctx, void *buffer,
 
 /* ------------------------------------------------------------------------- */
 
+const char *scanctx_getpath(struct scan_context *ctx)
+{
+  const char *name;
+  const char *full_path = NULL;
+
+  name = scanctx_take_string(ctx);
+  if((name[0] != FILE_SEPARATOR[0]) && ctx->config->include_dir)
+  {
+    full_path = scanctx_filename(ctx, ctx->config->include_dir, name);
+    free((void*)name);
+  }
+  else
+    full_path = name;
+
+  return full_path;
+}
+
+
+/* ------------------------------------------------------------------------- */
+
+const char *scanctx_filename(struct scan_context *ctx,const char* dirname, const char* filename)
+{
+  const char *basedir = dirname ? dirname : ctx->basedir;
+  char *full_file = (char *)malloc(strlen(basedir) + strlen(filename) + 2);
+
+  strcpy(full_file, basedir);
+  strcat(full_file, FILE_SEPARATOR);
+  strcat(full_file, filename);
+
+  return full_file;
+}
+
+/* ------------------------------------------------------------------------- */
+
 void *scanctx_pop_include(struct scan_context *ctx)
 {
   void *buffer;
@@ -146,13 +164,69 @@ void *scanctx_pop_include(struct scan_context *ctx)
 
   --(ctx->depth);
   buffer = ctx->buffers[ctx->depth];
-  fclose(ctx->streams[ctx->depth]);
+  if(ctx->streams[ctx->depth])
+    fclose(ctx->streams[ctx->depth]);
 
   return(buffer);
 }
 
 /* ------------------------------------------------------------------------- */
 
+extern const char *scanctx_dirnext(struct scan_context* ctx)
+{
+  struct dirent** dentries = (struct dirent**)ctx->dentries;
+  if(!ctx->dentries || (ctx->de_cur == ctx->de_max))	/* shouldn't happen.... */
+    return NULL;
+
+  return dentries[ctx->de_cur++]->d_name;
+}
+
+int scanctx_dirscan(struct scan_context* ctx, const char* dirname,
+                    int (*filter)(const struct dirent *),
+                    int (*compar)(const struct dirent **, const struct dirent **))
+{
+  int n;
+
+  if(!dirname)
+    return -1;
+
+  ctx->dentries = NULL;
+  if((n = scandir(dirname, (struct dirent***)&ctx->dentries, filter, compar)) < 0)
+    return n;
+
+  ctx->basedir = dirname;
+  ctx->de_max = n;
+  ctx->de_cur = 0;
+
+  return n;
+}
+
+void scanctx_dirend(struct scan_context* ctx)
+{
+  struct dirent **dentries = (struct dirent**)ctx->dentries;
+  unsigned i;
+
+  for(i = 0; i < ctx->de_max; i++)
+    free(dentries[i]);
+  free(ctx->dentries);
+  ctx->dentries = NULL;
+
+  if(ctx->basedir)
+    free((void*)ctx->basedir);
+
+  ctx->de_cur = ctx->de_max = 0;
+}
+
+int scanctx_inloop(const struct scan_context* ctx)
+{
+  if(!ctx->dentries)
+    return 0;
+
+  return (ctx->de_cur < ctx->de_max) ? 1 : 0;
+}
+
+/* ------------------------------------------------------------------------- */
+
 char *scanctx_take_string(struct scan_context *ctx)
 {
   char *r = strbuf_release(&(ctx->string));
